%DEMOHINF - Dieses Skript veranschaulicht den Entwurf und die Wirkungsweise
% eines H_unendlich-Reglers, welcher mittels des NLKF-Verfahrens entworfen
% wurde. 
%
% Autor: Martin Pfeifer
% Insitut fr Regelung und Steurungssysteme (IRS)
% Karlsruher Institut fr Technologie
% E-mail: martin.pfeifer@kit.edu 
% Website: http://www.irs.kit.edu
% Juni 2015; Letzte nderung: 07-Juni-2016 von Jona Maurer
%
%------------- BEGIN CODE --------------

%% Init
clc
clear all
close all

%% Lade und analysiere System

% 2x2 Systemmatrix G mit Y = G*U
s = tf('s');
% Systemordnung
n = 2;
I = eye(n);

% Teil-bertragungsfunktionen
g11 = 12 / (0.2*s + 1);
g12 = -0.05 / (0.1*s + 1);
g21 = 0.1 / (0.3*s + 1);
g22 = 5 / (0.7*s - 1);

% bertragungsmatrix
G = [g11, g12; g21, g22];

% Singulrwert-Diagramm der Strecke
figure(1)
% Erstelle Datenpunkte fr die x-Achse
omega = logspace(-1,3,100);
% Singulrwertdiagramm
sigma(G,omega)

%% Reglerentwurf
% Berechne H_unendlich Regler mit dem NLKF-Verfahren
[K_h,c1_h,gam] = ncfsyn(G);
K0 = -K_h;

% bertragungsfunktion Regler
tf(K0)
% bertragungsfunktion offener Kreis (ohne zustzlichen I-Anteil)
Fo = tf(G*K0);
% bertragungsfunktion offener Kreis (mit zustzlichen I-Anteil)
%Fo = tf(G*K0).*tf([1/s,1;1,1]);
% bertragungsfunktion des geschlossenen Kreises
CL = tf(Fo*(I+Fo)^-1);

%% Regelkreis-Analyse

% Berechne Kenngren des Regelkreises
% looptransfer = loopsens(G,K_h);

% Sensitivitt S=(I+G*K)^-1
S = (I+G*K0)^-1;
% komplementre Sensitivitt
T = (I+G*K0)^-1 * G*K0;

%% Darstellung der Ergebnisse

% Sprungantwort geschlossener Kreis
figure(2)
step(CL)

% Singulrwertdiagramm der Sensitivitten
figure(3)
hold on
% Sensitivitt
sigma(S,'b--',omega);
% Komplementre Sensitivitt
sigma(T,'r-.',omega)
% Legende
legend('S','T',3)

%% Erneuter Entwurf mit Gewichtungs-Filter
W1 = 20*(1/10^3*s + 1)/(1/10^2*s + 1)

% Berechne H_unendlich Regler mit dem NLKF-Verfahren unter der Gewichtung
[K_h1,c1_h1,gam] = ncfsyn(G,W1);
K1 = -K_h1;

%% Analyse und Darstellung Teil 2

% Berechne Kenngren des Regelkreises
% looptransfer = loopsens(G,K_h1);

% Sensitivitt S=(I+G*K)^-1
S1 = (I+G*K1)^-1;
% komplementre Sensitivitt
T1 = (I+G*K1)^-1 * G*K1;

%% Darstellung der Ergebnisse

% Bode-Diagramm der Filterfunktion
figure(4)
bode(W1)

% Singulrwertdiagramm der Sensitivitten
figure(5)
% Ohne Filter
subplot(1,2,1)
hold on
% Sensitivitt
sigma(S,'b--',omega);
% Komplementre Sensitivitt
sigma(T,'r-.',omega)
% Titel 
title('Ohne Gewichtungsmatrix')

% Mit Filter
subplot(1,2,2)
hold on
% Sensitivitt
sigma(S1,'b--',omega);
% Komplementre Sensitivitt
sigma(T1,'r-.',omega)
% Legende
legend('S','T',3)
% Titel 
title('Mit Gewichtungsmatrix')
